.normal-article-holder .color-blue-01 {
    color: #56c1fe
}

.normal-article-holder .color-lblue-01 {
    color: #73fdea
}

.normal-article-holder .color-green-01 {
    color: #89fa4e
}

.normal-article-holder .color-yellow-01 {
    color: #fff359
}

.normal-article-holder .color-pink-01 {
    color: #ff968d
}

.normal-article-holder .color-purple-01 {
    color: #ff8cc6
}

.normal-article-holder .color-blue-02 {
    color: #02a2ff
}

.normal-article-holder .color-lblue-02 {
    color: #18e7cf
}

.normal-article-holder .color-green-02 {
    color: #60d837
}

.normal-article-holder .color-yellow-02 {
    color: #fbe231
}

.normal-article-holder .color-pink-02 {
    color: #ff654e
}

.normal-article-holder .color-purple-02 {
    color: #ef5fa8
}

.normal-article-holder .color-blue-03 {
    color: #0176ba
}

.normal-article-holder .color-lblue-03 {
    color: #068f86
}

.normal-article-holder .color-green-03 {
    color: #1db100
}

.normal-article-holder .color-yellow-03 {
    color: #f8ba00
}

.normal-article-holder .color-pink-03 {
    color: #ee230d
}

.normal-article-holder .color-purple-03 {
    color: #cb297a
}

.normal-article-holder .color-blue-04 {
    color: #004e80
}

.normal-article-holder .color-lblue-04 {
    color: #017c76
}

.normal-article-holder .color-green-04 {
    color: #017001
}

.normal-article-holder .color-yellow-04 {
    color: #ff9201
}

.normal-article-holder .color-pink-04 {
    color: #b41700
}

.normal-article-holder .color-purple-04 {
    color: #99195e
}

.normal-article-holder .color-gray-01 {
    color: #d6d5d5
}

.normal-article-holder .color-gray-02 {
    color: #929292
}

.normal-article-holder .color-gray-03 {
    color: #5f5f5f
}

.normal-article-holder .color-default {
    color: #222
}
